#!/usr/bin/python

import os
import shutil
import csv
import itertools
import argparse
import ctypes
import re

def floatstrip(x):
  if x == int(x):
    return str(int(x))
  else:
    return str(x)


def createFileWithMeanValues(csvFile, inputDir, outputDir) :

    if (not os.access(outputDir, os.F_OK) ) :
        os.mkdir(outputDir)

    inputPath = inputDir
    inputPath += '/'
    inputPath += csvFile

    id = csvFile.replace ( '.csv', '' )

    if not os.path.exists(inputPath):
        print "File not found" + inputPath
        return

    noiseLevels = []
    radii = []

    with open(inputPath) as inputFile:
        dictOut = csv.DictReader(inputFile, delimiter=';')

        for row in dictOut:
            if float(row["Sigma noise"]) not in noiseLevels :
                noiseLevels.append(float(row["Sigma noise"]));
            if int(row["Radius"]) not in radii :
                radii.append(int(row["Radius"]));


    outFName = outputDir;
    outFName += id;
    outFName += '.csv';

    writer = csv.writer(open(outFName, 'wb'), delimiter=';')
    writer.writerow(["Radius", "Noise", "Repeatable Keypoints", "Non-occluded Keypoints", "Repeatability", "AbsoluteRepeatability", "ScaleRepeatability"])


    for noiseLevel in noiseLevels :
        for r in radii :
            with open(inputPath) as inputFile:
                dictOut = csv.DictReader(inputFile, delimiter=';')
                matchingKeypoint = 0
                notOccludedKeypoint = 0
                totScaleOverlap = 0
                nExperiments = 0

                for row in dictOut:
                    if float(row["Sigma noise"]) == noiseLevel and int(row["Radius"]) == r :
                        matchingKeypoint += int(row["Repeatable Keypoints"])
                        notOccludedKeypoint += int(row["Non-occluded Keypoints"])
                        totScaleOverlap += float(row["TotScaleRepeatability"])
                        nExperiments += 1
                        assert(totScaleOverlap <= matchingKeypoint)

            if not id.startswith('HKS') :
                writer.writerow([r, noiseLevel, matchingKeypoint, notOccludedKeypoint, matchingKeypoint*1.0/notOccludedKeypoint if notOccludedKeypoint > 0 else 0, matchingKeypoint*1.0/nExperiments if nExperiments > 0 else 0, totScaleOverlap*1.0/matchingKeypoint if matchingKeypoint > 0 else 0])
            else :
                for rr in [6,10,14,18]:
                 writer.writerow([rr, noiseLevel, matchingKeypoint, notOccludedKeypoint, matchingKeypoint*1.0/notOccludedKeypoint if notOccludedKeypoint > 0 else 0, matchingKeypoint*1.0/nExperiments if nExperiments > 0 else 0, totScaleOverlap*1.0/matchingKeypoint if matchingKeypoint > 0 else 0])



def createGNUPlotScriptRepeatabilityFixedScale(fileList, testId, inputDir, columnID, outputFilename):

    if (not os.access(inputDir, os.F_OK) ) :
        os.mkdir(inputDir)

    noiseLevels = []
    radii = []
    excludeFile = []

    for file in fileList :
        inputPath = inputDir
        inputPath += '/'
        inputPath += file

        with open(inputPath) as inputFile:
            dictOut = csv.DictReader(inputFile, delimiter=';')
            exclude = True
            for row in dictOut:
                if float(row["Noise"]) not in noiseLevels :
                    noiseLevels.append(float(row["Noise"]));
                if int(row["Radius"]) != -1 :
                    exclude = False
                if int(row["Radius"]) not in radii and int(row["Radius"]) != -1:
                    radii.append(int(row["Radius"]));
            excludeFile.append(exclude)


    for noiseLevel in noiseLevels:

      options = ' using 1:($2 == '+ str(noiseLevel) +' && $1 != -1 ? $'+ columnID+' : 1/0) with linespoints ls '

      s =  'set datafile separator ";" \n'
      s += 'set output "' + testId + '_fixed_'+ outputFilename.replace(" ", "") +'_Noise'+str(noiseLevel).replace(".", "_")+'.pdf" \n'
      s += 'set term pdf  dashed color font "Arial,8" enhanced \n'
      s += "set decimalsign '.'\n"
      #s += 'set term postscript eps dashed color blacktext "Times-Bold" 30 \n'
      if columnID != "6":
          s += 'set yrange [0:1]\n'
          s += 'set ytics 0,.1,1\n'
      s += 'set xrange [' + str(min(radii)) + ':' + str(max(radii)) + ']\n'
      s += 'set grid noxtics ytics\n'
      s += 'set key below center box horizontal width 1.2 on\n'
      s += 'set xlabel "Scale"\n'
      s += 'set ylabel "'+outputFilename+'"\n'
      s += 'set style line 2 lt 1 lw 5 pt 2  ps 1 linecolor rgb "red"\n'
      s += 'set style line 6 lt 1 lw 5 pt 4  ps 1 linecolor rgb "#32CD32"\n'
      s += 'set style line 4 lt 1 lw 5 pt 8  ps 1 linecolor rgb "blue"\n'
      s += 'set style line 5 lt 1 lw 5 pt 3  ps 1 linecolor rgb "magenta"\n'
      s += 'set style line 7 lt 1 lw 5 pt 6  ps 1 linecolor rgb "orange"\n'
      s += 'set style line 3 lt 1 lw 5 pt 12 ps 1 linecolor rgb "cyan"\n'
      s += 'set style line 1 lt 1 lw 5 pt 13 ps 0 linecolor rgb "#8B4513"\n'
      s += 'set style line 9 lt 1 lw 5 pt 14 ps 1 linecolor rgb "yellow"\n'
      s += 'set style line 8 lt 1 lw 5 pt 14 ps 1 linecolor rgb "black"\n'
      s += 'plot'
      first = True;

      for idx, file in enumerate(fileList):
        if excludeFile[idx] :
            continue
        file_tokens = re.split('\W+|_', file)
        s += (',' if not first else' ') + '"' + inputDir + '/' + file + '"' + options + str(idx+1) + ' title "'+ file_tokens[0]+'"'

        first = False

      s += '\nquit'

      fname = inputDir + '/plotter_fixed_'+outputFilename.replace(" ", "")+'_'+str(noiseLevel)+'.gpi'
      f = open(fname, 'w')
      f.write(s)
      f.close()
      cmd = '"gnuplot"'
      print cmd
      os.system(cmd + ' ' + fname)



def createGNUPlotScriptRepeatabilityAdaptiveScale(fileList, testId, inputDir, columnID, outputFilename):

    if (not os.access(inputDir, os.F_OK) ) :
        os.mkdir(inputDir)

    noiseLevels = []
    excludeFile = []

    for file in fileList :
        inputPath = inputDir
        inputPath += '/'
        inputPath += file

        with open(inputPath) as inputFile:
            dictOut = csv.DictReader(inputFile, delimiter=';')
            exclude = False
            for row in dictOut:
                if int(row["Radius"]) != -1 :
                    exclude = True
            excludeFile.append(exclude)


    #options = ' using 2:xtic(1) title col ls 4 fill pattern 1, \'\' u 5 ti col ls 5 fill pattern 2, \'\' u 8 ti col ls 6  fill pattern 3, \'\' u 11 ti col ls 8 fill pattern 4'#, \'\' u 14 ti col ls 9 fill pattern 5'#,  \'\' u 14 ti col ls 5 fill pattern 5'


    firstOptions = ' using '+columnID+':xticlabels(2) every ::1 fill pattern {0} ls ' #every ::1 skips the first line to have a correctly aligned x axis
    options = ' using '+columnID+' every ::1 fill pattern {0} ls '

    s =  'set datafile separator ";" \n'
    s += 'set output "' + testId + '_adaptive_'+ outputFilename.replace(" ", "") +'.pdf" \n'
    s += 'set term pdf  dashed color font "Arial,8" enhanced \n'
    s += "set decimalsign '.'\n"
    if columnID != "6":
        s += 'set yrange [0:1]\n'
        s += 'set ytics 0,.1,1\n'
    s += 'set style data histogram\n'
    s += 'set boxwidth 0.75 absolute\n'
    s += 'set style data histogram\n'
    s += 'set style histogram cluster gap 1\n'
    s += 'set style fill pattern border -1\n'
    s += 'set grid noxtics ytics\n'
    s += 'set xlabel "Noise"\n'
    s += 'set key below center box horizontal width 1.2 on\n'
    s += 'set ylabel "'+outputFilename+'"\n'
    s += 'set style line 2 lt 1 lw 5 pt 12 ps 1 linecolor rgb "red"\n'
    s += 'set style line 6 lt 1 lw 5 pt 12 ps 1 linecolor rgb "#32CD32"\n'
    s += 'set style line 4 lt 1 lw 5 pt 12 ps 1 linecolor rgb "blue"\n'
    s += 'set style line 5 lt 1 lw 5 pt 12 ps 1 linecolor rgb "magenta"\n'
    s += 'set style line 7 lt 1 lw 5 pt 12 ps 1 linecolor rgb "orange"\n'
    s += 'set style line 3 lt 1 lw 5 pt 12 ps 1 linecolor rgb "cyan"\n'
    s += 'set style line 1 lt 1 lw 5 pt 12 ps 0 linecolor rgb "#8B4513"\n'
    s += 'set style line 9 lt 1 lw 5 pt 12 ps 1 linecolor rgb "yellow"\n'
    s += 'set style line 8 lt 1 lw 5 pt 12 ps 1 linecolor rgb "black"\n'
    s += 'plot'
    first = True;


    fillPatternId = 1
    for idx, file in enumerate(fileList):
        if excludeFile[idx] :
            continue
        file_tokens = re.split('\W+|_', file)
        s += (',' if not first else' ') + '"' + inputDir + '/' + file + '"' + (options.format(fillPatternId) if not first else firstOptions.format(fillPatternId)) + str(idx+1) + ' title "'+ file_tokens[0]+'"'
        fillPatternId = fillPatternId + 1
        first = False

    s += '\nquit'

    fname = inputDir + '/plotter_adaptive_'+outputFilename.replace(" ", "")+'.gpi'
    f = open(fname, 'w')
    f.write(s)
    f.close()
    cmd = '"gnuplot"'
    print cmd
    os.system(cmd + ' ' + fname)


def run() :

    #parser = argparse.ArgumentParser(description='Creates the charts.')
    #parser.add_argument("path")
    #args = parser.parse_args()

    try:
        is_admin = ctypes.windll.shell32.IsUserAnAdmin()
    except:
        is_admin = True

    if not is_admin :
        print "You require admin privileges to run this script. Please run it from a prompt with admin privileges or use the provided batch file"
        raw_input('Press a key to exit')
        return

    testIds = ['StanfordViews', 'UWA', 'Retrieval', 'Kinect', 'SpaceTime']
    #locale.setlocale(locale.LC_ALL)

    for testId in testIds :
        print
        print
        print testId + " dataset"

        rootDir = '../results/' + testId + '/'
        outputDir = './'+testId+'/'



        if os.path.isdir(outputDir) :
            for the_file in sorted(os.listdir(outputDir)):
                file_path = os.path.join(outputDir, the_file)
                try:
                    if os.path.isfile(file_path):
                        os.unlink(file_path)
                except Exception, e:
                    print e

        if os.path.isdir(rootDir) :
            for file in sorted(os.listdir(rootDir)) :
                if file.endswith(".csv") :
                    print file
                    createFileWithMeanValues(file, rootDir, outputDir)
        else :
           print "No results for test " + testId
           continue

        filelist = [x for x in sorted(os.listdir(outputDir)) if x.endswith('.csv')]
        if os.path.isdir(outputDir) :
            createGNUPlotScriptRepeatabilityFixedScale(filelist, testId, outputDir, "5", "Relative Repeatability")
            createGNUPlotScriptRepeatabilityFixedScale(filelist, testId, outputDir, "6", "Absolute Repeatability")
            createGNUPlotScriptRepeatabilityAdaptiveScale(filelist, testId, outputDir, "5", "Relative Repeatability")
            createGNUPlotScriptRepeatabilityAdaptiveScale(filelist, testId, outputDir, "6", "Absolute Repeatability")
            createGNUPlotScriptRepeatabilityAdaptiveScale(filelist, testId, outputDir, "7", "Scale Repeatability")


        #createFilesWithMeanValuesForScaleSpace("Radius", radius, rootDir, createHebertFilenames(Scales, FirstScale, OctavesDistance, BorderDistance, minNeigh) , outputDir, "Hebert", noiselevels)
        #createFilesWithMeanValuesForScaleSpace("Radius", radius, rootDir, createMeshDogFilenames(Scales, FirstScale, FeatType, KeypPerc, CornerTh), outputDir, "MeshDoG", noiselevels)
        #createFilesWithMeanValuesForScaleSpace("Radius", radius, rootDir, createMianWithScaleFilenames(Scales, FirstScale, OctavesDistance, RNonMaxima, GridSide, minNeigh), outputDir, "MianScale", noiselevels)

        #createGNUPlotScriptAbsoluteRepeatability(outputDirFigures, outputDir, radius, noiselevels, testId )
        #createGNUPlotScriptScaleRepeatability(outputDirFigures, outputDir, noiselevels, testId )

      #createGNUPlotScriptDetectionTime(outputDirFigures, paperHome + '/results/testing/timingsRetrieval.csv', 'Retrieval' )
      #createGNUPlotScriptDetectionTime(outputDirFigures, paperHome + '/results/testing/timingsKinect.csv', 'Kinect' )

    raw_input('Press a key to exit')

if __name__ == '__main__' :
    run()
